@@ -6,59 +6,6 @@
 # mse_estim (see below)
 # The parametric boostrap approach can be find in Molina and Rao (2010) p. 376
 
-# External documentation -------------------------------------------------------
-
-# #' Estimates the Parametric Bootstrap MSE of the EBP approach
-# #'
-# #' Function \code{parametric_bootstrap} estimates the parametric bootstrap MSE
-# #' for regional disaggregated indicators using the Parametric Bootstrap approach
-# #' by \cite{Gonzalez-Manteiga et al. (2008)}.
-# #'
-# #' @param framework the result of applying function \code{\link{notation}}. A
-# #' list with 15 components containing rearranged data sets and vectors of
-# #' variables indicating the domains as well as determinants of the notational
-# #' framework (see also \code{\link{emdiObject}}).
-# #' @param point_estim the result of applying \code{\link{point_estim}}.
-# #' @param fixed fixed a two-sided linear formula object describing the
-# #' fixed-effects part of the nested error linear regression model with the
-# #' dependent variable on the left of a ~ operator and the explanatory
-# #' variables on the right, separated by + operators. The argument corresponds
-# #' to the argument \code{fixed} in function \code{\link[nlme]{lme}}.
-# #' @param transformation a character string. Three different transformation
-# #' types for the dependent variable can be chosen (i) no transformation ("no");
-# #' (ii) log transformation ("log"); (iii) Box-Cox transformation ("box.cox").
-# #' @param interval a numeric vector containing a lower and upper limit
-# #' determining an interval for the estimation of the optimal parameter. Defaults
-# #' to c(-1,2).
-# #' @param L a number determining the number of Monte-Carlo simulations.
-# #' @param B a number determining the number of bootstrap populations in the
-# #' parametric bootstrap approach (see also \cite{Gonzalez-Manteiga et al. (2008)})
-# #' used in the MSE estimation.
-# #' @return a data frame with MSE estimators for each indicator per domain.
-# #' @references
-# #' Gonzalez-Manteiga, W. et al. (2008). Bootstrap mean squared error of
-# #' a small-area EBLUP. Journal of Statistical Computation and Simulation,
-# #' 78:5, 443-462.
-# #' @seealso \code{\link{ebp}}, \code{\link{emdiObject}}, \code{\link{notation}},
-# #' \code{\link{point_estim}}
-# #' @examples
-# #' # Loading data
-# #' data("Xoutsamp_AuxVar")
-# #' data("incomedata")
-# #'
-# #' # Determine notational framework
-# #' framework <- notation(Xoutsamp_AuxVar, "provlab", incomedata, "provlab")
-# #'
-# #' # Obtain point predictions
-# #' set.seed(100); point <- point_estim(framework, income~educ1, 4500,
-# #' "box.cox", L=2)
-# #'
-# #' # Parametric bootstrap MSE
-# #' set.seed(100); mse <- parametric_bootstrap(framework, point, income ~ educ1,
-# #' 4500, "box.cox", L=2, B=2)
-# #' @export
-# #' @import nlme
-
 parametric_bootstrap <- function(framework,
                                  point_estim,
                                  fixed,
@@ -66,10 +13,18 @@ parametric_bootstrap <- function(framework,
                                  interval=c(-1,2),
                                  L,
                                  B,
+                                 boot_type,
                                  parallel_mode,
                                  cpus) {
   cat('\r', "Bootstrap started                                                                        ")
-  
+  if(boot_type == "wild"){
+    res_s <- residuals(point_estim$model)
+    fitted_s <- fitted(point_estim$model, level = 1)
+  } else {
+    res_s <- NULL
+    fitted_s <- NULL
+  }
+
   start_time = Sys.time()
   if(cpus > 1){
     cpus <- min(cpus, detectCores())
@@ -87,7 +42,10 @@ parametric_bootstrap <- function(framework,
                                            transformation  = transformation,
                                            interval        = interval,
                                            L               = L,
-                                           start_time      = start_time
+                                           res_s           = res_s,
+                                           fitted_s        = fitted_s,
+                                           start_time      = start_time,
+                                           boot_type       = boot_type
                   )
             )
     parallelStop()
@@ -104,7 +62,10 @@ parametric_bootstrap <- function(framework,
                                             transformation  = transformation,
                                             interval        = interval,
                                             L               = L,
-                                            start_time      = start_time
+                                            res_s           = res_s,
+                                            fitted_s        = fitted_s,
+                                            start_time      = start_time,
+                                            boot_type       = boot_type
         )
       )
   }
@@ -113,7 +74,7 @@ parametric_bootstrap <- function(framework,
   if(.Platform$OS.type == "windows"){
     flush.console()
   }
-  mses <- apply(mses, c(1,2), fast_mean)
+  mses <- apply(mses, c(1,2), mean)
   mses <- data.frame(Domain = unique(framework$pop_domains_vec), mses)
 
   return(mses)
@@ -133,10 +94,13 @@ mse_estim <- function(framework,
                       shift,
                       model_par,
                       gen_model,
+                      res_s,
+                      fitted_s,
                       fixed,
                       transformation,
                       interval,
-                      L
+                      L,
+                      boot_type
                       ) {
 
 
@@ -144,28 +108,46 @@ mse_estim <- function(framework,
   # The function superpopulation returns an income vector and a temporary
   # variable that passes the random effect to generating bootstrap populations
   # in bootstrap_par.
-  superpop <- superpopulation(framework      = framework,
-                              model_par      = model_par,
-                              gen_model      = gen_model,
-                              lambda         = lambda,
-                              shift          = shift, 
-                              transformation = transformation)
-
+  
+  if(boot_type == "wild"){
+    superpop <- superpopulation_wild(framework      = framework,
+                                      model_par      = model_par,
+                                      gen_model      = gen_model,
+                                      lambda         = lambda,
+                                      shift          = shift, 
+                                      transformation = transformation,
+                                      res_s          = res_s,
+                                      fitted_s       = fitted_s
+    )
+  }
+  else{
+    superpop <- superpopulation(framework      = framework,
+                                model_par      = model_par,
+                                gen_model      = gen_model,
+                                lambda         = lambda,
+                                shift          = shift, 
+                                transformation = transformation
+                               )
+  }
   pop_income_vector <- superpop$pop_income_vector
-
+  
+  if("function" %in% class(framework$threshold )){
+    framework$threshold <- 
+      framework$threshold(y = pop_income_vector)
+  }
   # True indicator values
   true_indicators <- matrix(nrow = framework$N_dom_pop,
                             data = unlist(lapply(framework$indicator_list,
-                                   function(f, pov_line){
+                                   function(f, threshold){
                                      matrix(nrow = framework$N_dom_pop,
                                             data = unlist(tapply(pop_income_vector,
                                                                  framework$pop_domains_vec, f,
-                                                                 pov_line = framework$pov_line ,
+                                                                 threshold = framework$threshold ,
                                                                  simplify = TRUE)
                                                           ),
                                             byrow=TRUE)
                                      },
-                                   pov_line = framework$pov_line)
+                                   threshold = framework$threshold)
                                    )
                             )
 
@@ -174,15 +156,28 @@ mse_estim <- function(framework,
   # The function bootstrap_par returns a sample that can be given into the
   # point estimation to get predictors of the indicators that can be compared
   # to the "truth".
-  bootstrap_sample <- bootstrap_par(fixed          = fixed,
-                                    transformation = transformation,
-                                    framework      = framework,
-                                    model_par      = model_par,
-                                    lambda         = lambda,
-                                    shift          = shift,
-                                    vu_tmp         = superpop$vu_tmp
-                                    )
-
+  
+  if(boot_type == "wild"){  
+    bootstrap_sample <- bootstrap_par_wild(fixed          = fixed,
+                                      transformation = transformation,
+                                      framework      = framework,
+                                      model_par      = model_par,
+                                      lambda         = lambda,
+                                      shift          = shift,
+                                      vu_tmp         = superpop$vu_tmp,
+                                      res_s          = res_s,
+                                      fitted_s       = fitted_s
+    )
+  }else{
+    bootstrap_sample <- bootstrap_par(fixed          = fixed,
+                                      transformation = transformation,
+                                      framework      = framework,
+                                      model_par      = model_par,
+                                      lambda         = lambda,
+                                      shift          = shift,
+                                      vu_tmp         = superpop$vu_tmp
+    )
+  }
   framework$smp_data <- bootstrap_sample
 
   # Prediction of indicators with bootstap sample.
@@ -201,6 +196,38 @@ mse_estim <- function(framework,
 
 # The model parameter from the nested error linear regression model are
 # used to contruct a superpopulation model.
+superpopulation_wild <-  function(framework, model_par, gen_model, lambda, shift, 
+                             transformation, res_s,  fitted_s) {
+  # rescaling the errors
+  res_s <- sqrt(model_par$sigmae2est) * (res_s - mean(res_s))/sd(res_s)
+  
+  # superpopulation random effect
+  vu_tmp <- rnorm(framework$N_dom_pop, 0, sqrt(model_par$sigmau2est))
+  vu_pop <- rep(vu_tmp, framework$n_pop)
+  
+  # income without individual errors
+  Y_pop_b <- gen_model$mu_fixed + vu_pop
+  
+  indexer <- sapply(Y_pop_b, function(x) {which.min(abs(x - fitted_s))})
+  
+  # superpopulation individual errors
+  eps <- res_s[indexer]
+  wu <- sample(c(-1,1),size = length(eps), replace = T)
+  eps <- abs(eps) * wu
+  
+  #  superpopulation income vector
+  Y_pop_b <- Y_pop_b + eps
+ 
+  Y_pop_b <- back_transformation(y              = Y_pop_b,
+                                 transformation = transformation,
+                                 lambda         = lambda,
+                                 shift          = shift
+  )
+  Y_pop_b[!is.finite(Y_pop_b)] <- 0
+  
+  return(list(pop_income_vector = Y_pop_b, vu_tmp = vu_tmp))
+}
+
 superpopulation <-  function(framework, model_par, gen_model, lambda, shift, 
                              transformation) {
   # superpopulation individual errors
@@ -260,6 +287,44 @@ bootstrap_par <- function(fixed,
   return(bootstrap_sample = bootstrap_smp)
 }
 
+bootstrap_par_wild <- function( fixed,
+                                transformation,
+                                framework,
+                                model_par,
+                                lambda,
+                                shift,
+                                vu_tmp,
+                                res_s,  
+                                fitted_s) {
+  # rescaling sample individual error term
+  res_s <- sqrt(model_par$sigmae2est) * (res_s - mean(res_s))/sd(res_s)
+  # Bootstrap sample individual error term
+  ws <- sample(c(-1,1),size = length(res_s), replace = T)
+  eps <- abs(res_s) * ws
+  
+  # Bootstrap sample random effect
+  vu_smp <- rep(vu_tmp[framework$dist_obs_dom], framework$n_smp)
+  
+  # Extraction of design matrix
+  X_smp <- model.matrix(fixed, framework$smp_data)
+  
+  # Transformed bootstrap income vector
+  Y_smp_b <- X_smp %*% model_par$betas + eps + vu_smp
+  
+  # Back transformation of bootstrap income vector
+  Y_smp_b <- back_transformation(y              = Y_smp_b,
+                                 transformation = transformation,
+                                 lambda         = lambda,
+                                 shift          = shift
+  )
+  Y_smp_b[!is.finite(Y_smp_b)] <- 0
+  
+  # Inclusion of bootstrap income vector into sample data
+  bootstrap_smp <- framework$smp_data
+  bootstrap_smp[paste(fixed[2])] <- Y_smp_b
+  
+  return(bootstrap_sample = bootstrap_smp)
+}
 
 # progress for mse_estim (only internal) ----------
 
@@ -274,17 +339,23 @@ mse_estim_wrapper <-  function(i,
                                transformation,
                                interval,
                                L,
-                               start_time) {
+                               res_s,
+                               fitted_s,
+                               start_time,
+                               boot_type) {
 
   tmp <- mse_estim(framework       = framework,
                    lambda          = lambda,
                    shift           = shift,
                    model_par       = model_par,
                    gen_model       = gen_model,
+                   res_s           = res_s,
+                   fitted_s        = fitted_s, 
                    fixed           = fixed,
                    transformation  = transformation,
                    interval        = interval,
-                   L               = L
+                   L               = L, 
+                   boot_type       = boot_type
                    )
 
   if(i%%10 == 0) {
@@ -316,6 +387,20 @@ mse_estim_wrapper <-  function(i,
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
